/***********************************************************************
  sudokuSolver.c

  Apr 23, 2009: zhuo

  a test main app for sudokuSolverCore*.c module.

  usage

  % sudokuSolver001  < (init_text) > (result_text)

  the text format is as follows:

     000012345
     678901020
     304050607
     :
     123465789

  0 represents an unfilled block, while 1-9 are for fixed initial values.
  "CR"s are ignored, so the input shown above is identical to:
     000012345678901020304050607 ... 123465789
  or, even to:
     0000
     12345678
     90
     1020304050607
     :
     1
     23465789
   Thus, the system starts solving every 81 digits of input.
   At maximum, 128 characters are accepted in a single line.

   Note, however, that a blank line terminates the application.
     
***********************************************************************/

#include <stdio.h>
#include <string.h>





#include "sudokuSolverCore.h"

#define BUF_SIZE 128





void dumpValues( int*values ) {
  int x;
  int y;
  for ( y = 0; y < 9; ++y ) {
    for ( x = 0; x < 9; ++x ) {
      printf( "%1d", *values++ );
    }
    printf( "\n" );
  }
}



int main( int argc, char** argv ) {
  static char buf[ BUF_SIZE ];
  static int values[ BUF_SIZE ];
  char* p;
  int idx = 0;
  int result;

  SSInitialize();

  while ( fgets( buf, BUF_SIZE, stdin ) != NULL && strlen( buf ) > 1 ) {
    for ( p = buf; *p != '\n'; ++p ) {
      values[ idx ] = (int)( *p - '0' );
      if ( ++idx == 81 ) {
	SSLoadFromIntArray( values );
	result = SSSolve();
	fprintf( stderr, "%s\n", result ? "successful" : "failed" );
	SSStoreToIntArray( values );
	dumpValues( values );
	idx = 0;
      }
    }
  }
  return 0;
}

