/***********************************************************************
  sudokuSolverCore002.c
  0.01 : Apr. 22, 2009  zhuo  -- "reduce" done.
  0.02 : May. 5, 2009  zhuo -- "infer" done.
***********************************************************************/

#include "sudokuSolverCore.h"




#ifdef VERBOSE
#include <stdio.h>
#define PRINT( ARGS ) printf ARGS
#else
#define PRINT( ARGS ) 
#endif





/*****************************************
  TYPES, SYMBOLS, TABLES, UTILITY FUNCTIONS
*****************************************/
typedef struct {
    int bitValue;
    int nCand;
} SS;

#define SS_N 9
#define SS_NN (SS_N * SS_N)

#define SS_B0 1
#define SS_B1 (1<<1)
#define SS_B2 (1<<2)
#define SS_B3 (1<<3)
#define SS_B4 (1<<4)
#define SS_B5 (1<<5)
#define SS_B6 (1<<6)
#define SS_B7 (1<<7)
#define SS_B8 (1<<8)
#define SS_B9 (1<<9)
#define SS_B1_B9 (SS_B1|SS_B2|SS_B3|SS_B4|SS_B5|SS_B6|SS_B7|SS_B8|SS_B9)


static const int SSBitMask[] = {
  SS_B0, SS_B1, SS_B2, SS_B3, SS_B4, SS_B5, SS_B6, SS_B7, SS_B8, SS_B9
};

#define SS_SIZE_OF_BITMASK  ((signed int)(sizeof(SSBitMask)/sizeof(int)))






static int SSBitToNum( int pattern, int searchFrom ) {
  int i = searchFrom;
  if ( pattern == 0 ) {
    return 0;
  }
  while ( i < SS_SIZE_OF_BITMASK ) {
    if ( SSBitMask[ i ] & pattern ) {
      return i;
    }
    ++i;
  }
}





/*********************
  a simple queue
  to hold the indice P
  for recursion
  P = 9*y+x
*********************/
typedef struct {
  int indice[ SS_NN ];  /* twice the size of SS_NN to hold x and y */
  int *pRead;
  int *pWrite;
  int cnt;
} SS_QUEUE;

/*
  A queue for Reduce  (with declaration )
 */
static SS_QUEUE SSReducedIndice;

static void SSQueueInit( void ) {
  SS_QUEUE* pQ = &SSReducedIndice;
  pQ->pRead = &pQ->indice[0];
  pQ->pWrite = &pQ->indice[0];
  pQ->cnt = 0;
}

static void SSQueueAdd( int index ) {
  SS_QUEUE* pQ = &SSReducedIndice;
  if ( pQ->cnt < SS_NN ) {
    *(pQ->pWrite)++ = index;
    ++ pQ->cnt;
  }
}

/*
  return the index when anything remain.
  -1 when the queue is empty.
*/
static int SSQueueTake() {
  int value;
  SS_QUEUE* pQ = &SSReducedIndice;
  if ( pQ->cnt > 0 ) {
    --pQ->cnt;
    value = *(pQ->pRead);
    ++(pQ->pRead);
    return value;
  }
  return -1;
}






/*****************************************
  Panel Data and Save Stack
*****************************************/

/* 
  data[ a single panel, in 1D array]
*/
static SS SSPanelData[ SS_NN ];
static void SSPanelCopy( SS* dst, SS* src ) {
  int idx;
  for ( idx = 0; idx < SS_NN; ++idx, ++src, ++dst ) {
    dst->bitValue = src->bitValue;
    dst->nCand = src->nCand;
  }
}





/************************************************************
    API IMPLEMENTATION
 ************************************************************/
void SSInitialize( void ) {
  /* nothing to do, so far */
}





/* load values[81] (0 ... blank, 1-9: fixed value) to paneldata */
void SSLoadFromIntArray( int* values ) {
  int idx;
  for ( idx = 0; idx < SS_NN; ++idx ) {
    PRINT ((  "values[%2d]=%d: ", idx, values[idx]  ));
    if ( values[idx] == 0 ) {
      SSPanelData[idx].bitValue = SS_B1_B9;
      SSPanelData[idx].nCand = 9;
    } else {
      SSPanelData[idx].bitValue = SSBitMask[ values[idx] ];
      SSPanelData[idx].nCand = 1;
    }
  }
}




void SSStoreToIntArray( int* values ) {
  int idx;
  for ( idx = 0; idx < SS_NN; ++idx ) {
    if ( SSPanelData[idx].nCand == 1 ) {
      values[idx] = SSBitToNum( SSPanelData[idx].bitValue, 0 );
    } else {
      values[idx] = 0;
    }
  }
}




/*****************************************
  Reduction
 ****************************************/
static int SSReduceElem(SS* data, int idx, int valPattern ) {
  int bit = data[idx].bitValue;
  int n;
  if ( bit & valPattern ) {
    data[idx].bitValue = bit & (~valPattern);
    n = --data[idx].nCand;
    if ( n == 0 ) {
      return 0;
    }
    if ( n == 1 ) {
      SSQueueAdd( idx );
    }
  }
  return 1;
}

/*
  returns -1  for failure (conflict, etc)
  returns N  for  # successful reductions
 */
static int SSReduceIndex( SS* data, int idx ) {
  int valPattern;
  int bx, by, xfrom, yfrom;
  int x, y, h, v;
  /*
    +++|+++++   (1) '#' : find corner, and erase
    +++|+++++             boxes (except itself)
    +++|+++++   (2) '|' : erase the vert line (but not the # box)
    +++|##+++   (3) '=':  erace the horiz line (but not the # box)
    ===o=====
    +++|##+++
    +++|+++++
    +++|+++++
    +++|+++++
  */
  valPattern = data[ idx ].bitValue;
  if ( valPattern == 0x00 ) {
    return -1;
  }

  bx = idx % 9;
  by = idx / 9;
  xfrom = (bx / 3) * 3;
  yfrom = (by / 3) * 3;
  for ( y = yfrom ; y < yfrom + 3; ++y ) {
    if ( y == by ) { continue; }
    for ( x = xfrom; x < xfrom + 3; ++x ) {
      if ( x == bx ) { continue; }
      if ( SSReduceElem( data, (y*9+x), valPattern ) == 0 ) {
	return -1;
      }
    }
  }
  
  for ( v = bx, h = idx - bx; v < SS_NN; v += 9, ++h ) {
    if ( v != idx ) {
      if ( SSReduceElem( data, v, valPattern ) == 0 ) {
	return -1;
      }
    }
    if ( h != idx ) {
      if ( SSReduceElem( data, h, valPattern ) == 0 ) {
	return -1;
      }
    }
  }
  return 0;
}



/*
  returns -1  for failure (conflict, etc)
  returns N  for  # successful reductions
 */
static int SSReduceIndice( SS* data ) {
  int idx;
  int cnt = 0;
  while ( (idx = SSQueueTake()) != -1 ) {
    /* 
       try reduction, give up and return  if error/conflict is raised.
       Note that this reduction may add extra indice  for further
       reduction.
    */
    PRINT (( "reducing [%d]:", idx ));
    if ( SSReduceIndex( data, idx ) == -1 ) {
      PRINT (( "fail\n" ));
      return -1;
    }
    PRINT (( "good\n" ));
    ++cnt;
  }
  return cnt;
}



/*
  This reduces the indice in the queue  on the data.
  If new indice are found, they will be appended to the queue
  and be processed as well.

  This returns -1  for failure (conflict, etc)
  This returns N  for  # successful reductions
 */
static int SSReduce( SS* data ) {
  int idx;
  SSQueueInit();
  for ( idx = 0; idx < SS_NN; ++idx ) {
    if ( data[idx].nCand == 1 ) {
      SSQueueAdd( idx );
    }
  }
  return SSReduceIndice( data );
}


/*
######################################################################
*/
/*
  findFirstMultipe:
  0 : not found
  -1:  repo
 */
static int SSFindFirstMultiple ( SS* data ) {
  int idx;
  for ( idx = 0; idx < SS_NN; ++idx ) {
    if ( data[idx].nCand > 1 ) {
      return idx;
    }
  }
  return -1;
}





/*
  0 : fail
  1 : successful
*/
static int SSInfer( SS* data ) {
  int idx = SSFindFirstMultiple( data );
  int bitValue;
  int cand;
  SS backup[ SS_NN ];

  if ( idx == -1 ) {
    return 1;
  }

  PRINT (( "SSInfer : " ));
  // save the current panel
  SSPanelCopy( backup, data );
  bitValue = data[ idx ].bitValue;
  PRINT (( "looking at data[%d] = %04x\n", idx, bitValue  ));
  for ( cand = 1; cand < 10; ++cand ) {
    if ( bitValue & (SSBitMask[ cand ]) ) {
      PRINT(( "trying %d : ", cand ));
      data[ idx ].bitValue = SSBitMask[ cand ];
      data[ idx ].nCand = 1;
      SSQueueInit();
      SSQueueAdd( idx );
      if ( SSReduceIndice( data ) != -1 ) {
        PRINT (( "reduction good - next level\n" ));
        if ( SSInfer( data ) == 1 ) {
	  return 1;
        }
      } else {
        PRINT (( "reduction fail - next choice\n" ));
      }
      SSPanelCopy( data, backup );
    }
  }
  PRINT (( "reduction fail - prev level\n" ));
  return 0;
}


/* solve. 
   initial values are read from SSPanelData[81], and 
   the results are stored back in SSPanelData[81].
   returns : 
   0 when failed
   1 when successful
*/
int SSSolve( void ) {
  if ( SSReduce( SSPanelData ) == SS_NN ) {
    return 1;
  }
  return SSInfer( SSPanelData );
}
